class ApocHud extends GameHUD;

var bool bDrawTraces; //Hold exec console function switch to display debug of trace lines & Paths.
var FontRenderInfo TextRenderInfo; //Font for outputted text to viewport

//fetch mouse coordinates from the UI Hierarchy in the UIController of the PlayerController
function vector2D GetMouseCoordinates()
{
	local Vector2D mousePos;
	local UIInteraction UIController;
	local GameUISceneClient GameSceneClient;
	UIController = PlayerOwner.GetUIController();
	if(UIController != none)
	{
		GameSceneClient = UIController.SceneClient;
		if(GameSceneClient != none)
		{
			mousePos.X = GameSceneClient.MousePosition.X;
			mousePos.Y = GameSceneClient.MousePosition.Y;
		}
	}
	return mousePos;
}

event PostRender()
{
	local IsometricCamera playerCam;
	local ApocPlayerController aPlayerController;
	super.PostRender();

	aPlayerController = ApocPlayerController(PlayerOwner);
	aPlayerController.PlayerMouse = GetMouseCoordinates();

	//Deproject 2d mouse coordinate into 3d world
	Canvas.DeProject(aPlayerController.PlayerMouse, aPlayerController.MousePosWorldLocation, aPlayerController.MousePosWorldNormal);
	playerCam = IsometricCamera(aPlayerController.PlayerCamera);
	
	//Calculate a trace from Player camera + 100 up(z) in direction of deprojected MousePosWorldNormal (the direction of the mouse).
	
	//Set the ray direction as the mouseWorldnormal
	aPlayerController.RayDir = aPlayerController.MousePosWorldNormal;
	
	//Start the trace at the player camera (isometric) + 100 unit z and a little offset in front of the camera (direction *10)
	aPlayerController.StartTrace = (playerCam.ViewTarget.POV.Location + vect(0,0,100)) + aPlayerController.RayDir * 10;
	
	//End this ray at start + the direction multiplied by given distance (5000 unit is far enough generally)
	aPlayerController.EndTrace = aPlayerController.StartTrace + aPlayerController.RayDir * 5000;
	
	//Trace MouseHitWorldLocation each frame to world location (here you can get from the trace the actors that are hit by the trace, for the sake of this
	//simple tutorial, we do noting with the result, but if you would filter clicks only on terrain, or if the player clicks on an npc, you would want to inspect
	//the object hit in the StartFire function
	aPlayerController.TraceActor = Trace(aPlayerController.MouseHitWorldLocation, aPlayerController.MouseHitWorldNormal, 
										aPlayerController.EndTrace, aPlayerController.StartTrace,true);
	aPlayerController.PawnEyeLocation = Pawn(PlayerOwner.ViewTarget).Location + Pawn(PlayerOwner.ViewTarget).EyeHeight * vect(0,0,1);

	DrawHUD();
	if(bDrawTraces)
	{
		super.DrawRoute(Pawn(PlayerOwner.ViewTarget));
		DrawTraceDebugRays();
	}
}

/******************************************************************
 *  TUTORIAL FUNCTION  
 *  
 *  Declare a new console command to control debug of 3d line
 *  debug drawing. This will also control of showing the paths
 *  the pawn will have available into its calculated routes.
 *  
 *  ******************************************************************/
exec function ToggleIsometricDebug()
{
	bDrawTraces = !bDrawTraces;
	if(bDrawTraces)
	{
		`Log("Showing debug line trace for mouse");
	}
	else
	{
		`Log("Disabling debug line trace for mouse");
	}
}

function DrawTraceDebugRays()
{
	local ApocPlayerController aPlayerController;
	aPlayerController = ApocPlayerController(PlayerOwner);
	//Line from camera to the world
	Draw3DLine(aPlayerController.StartTrace, aPlayerController.EndTrace, MakeColor(255,128,128,255));
	//Line from eye to determine collision
	Draw3DLine(aPlayerController.PawnEyeLocation, aPlayerController.MousePosWorldLocation, MakeColor(0,200,255,255));
}

function DrawDebugCross(Vector pos, float size)
{
	local Vector startX;
	local Vector endX;
	
	startX = pos;
	startX.X -= size;
	endX = pos;
	endX.X += size;
	Draw3DLine(startX, endX, MakeColor(255, 0, 0, 255));

	startX = pos;
	startX.Y -= size;
	endX = pos;
	endX.Y += size;
	Draw3DLine(startX, endX, MakeColor(0, 255, 0, 255));

	startX = pos;
	startX.Z -= size;
	endX = pos;
	endX.Z += size;
	Draw3DLine(startX, endX, MakeColor(0, 0, 255, 255));
}

function DrawHud()
{
	local string StringMsg;
	local ApocPlayer PlayerPawn;

	PlayerPawn = ApocPlayer(ApocPlayerController(PlayerOwner).Pawn);

	if(ApocPlayerController(PlayerOwner).TraceActor != none)
	{
		StringMsg = "Actor selected:"@ApocPlayerController(PlayerOwner).TraceActor.Class;
	}
	Canvas.DrawColor = MakeColor(255,183,11,255);
	Canvas.SetPos(250,10);
	Canvas.DrawText(StringMsg, false, , , TextRenderInfo);
	StringMsg = "Playing shit?" @ PlayerPawn.IsPlayingCustomAnim('Death_attack');
	Canvas.SetPos(250,20);
	Canvas.DrawText(StringMsg, false, , , TextRenderInfo);

	//Canvas.DrawColor = MakeColor(0, 0, 255, 255);
	//StringMsg = "Pawn Loc:" @ PlayerPawn.Location @ "BlockVolume Loc:" @ PlayerPawn.BlockVolume.Location;
	//Canvas.SetPos(250,30);
	//Canvas.DrawText(StringMsg, false, , , TextRenderInfo);

}

DefaultProperties
{
}