class ApocInteractiveFoliageActor extends Actor implements(ApocTargetableInterface, ApocNavPointInterface);

struct NavPoint
{
	var Vector navPoint;
	var bool isOccupied;
};

var(Apocalypse) InteractiveFoliageComponent Mesh;
var(Apocalypse) array<NavPoint> NavPoints;

simulated event PostBeginPlay()
{
	super.PostBeginPlay();
	OnPostBeginPlay_AddToTargetableList();
}

simulated event Destroyed()
{
	super.Destroyed();
	OnDestroy_RemoveFromTargetableList();
}

function Vector GetNavPoint()
{
	local NavPoint currentPoint;

	//`log("There are" @ NavPoints.Length @ "navpoints");
	if(NavPoints.Length == 0)
	{
		return Location;
	}
	foreach NavPoints(currentPoint)
	{
		if(!currentPoint.isOccupied)
		{
			return currentPoint.navPoint;
		}
		else
		{
			//`log("Navpoint" @ currentPoint.navPoint @ "is occupied");
		}
	}
	return NavPoints[Rand(NavPoints.Length)].navPoint;

}

function Vector GetNavPointAt(int index)
{
	return NavPoints[index].navPoint;
}

/** Adds a Target to the global targets array in GameInfo, upon creation/PostBeginPlay */
function OnPostBeginPlay_AddToTargetableList()
{
	main(WorldInfo.Game).TargetableActors.AddItem(self);
}

/** Removes a target from the global targets array in GameInfo, upon death/destruction */
function OnDestroy_RemoveFromTargetableList()
{
	main(WorldInfo.Game).TargetableActors.RemoveItem(self);
}

DefaultProperties
{
	Begin Object Class=DynamicLightEnvironmentComponent Name=MyLightEnvironment
              AmbientGlow=(R=0.0,G=0.0,B=0.0,A=1.0)
              AmbientShadowColor=(R=0.0,G=0.0,B=0.0)
              ShadowFadeResolution=16
              MinShadowResolution=8
              BouncedLightingFactor=0.0
              bSynthesizeSHLight=false
              LightDistance=6
              ShadowDistance=6
              ModShadowFadeoutExponent=1
              bCastShadows=true
              ShadowFilterQuality=SFQ_Medium
			  MinTimeBetweenFullUpdates = 0.3
			  bCompositeShadowsFromDynamicLights=false
              bDynamic=true
              MinShadowAngle=0
    End Object

	Components.Add(MyLightEnvironment);

	Begin Object Class=InteractiveFoliageComponent Name=InitialStaticMesh
		CastShadow=true
		bOwnerNoSee=false
		LightEnvironment=MyLightEnvironment;
        BlockRigidBody=true;
        CollideActors=true;
		StaticMesh=StaticMesh'Apo_static.houses.APO_well01'
	End Object
	CollisionComponent = InitialStaticMesh;

	Mesh = InitialStaticMesh;
	Components.Add(InitialStaticMesh);
}