class ApocPawn extends UDKPawn implements(ApocTargetableInterface);

const AVOID_DISTANCE = 300.0f;
const MAX_PAWN_SPEED = 600.0f;

var(Apocalypse) ApocWeapon WeaponTemplate;
var(Apocalypse) DynamicLightEnvironmentComponent LightEnvironment;
var(Apocalypse) Name CustomAnimNodeSlotName;
var AnimNodePlayCustomAnim CustomAnimNodeSlot;

defaultproperties
{
	bIsWalking = true;

	Components.Remove(Sprite)

	Begin Object Class=DynamicLightEnvironmentComponent Name=MyLightEnvironment
		ModShadowFadeoutTime=0.25
		MinTimeBetweenFullUpdates=0.2
		AmbientGlow=(R=.01,G=.01,B=.01,A=1)
		AmbientShadowColor=(R=0.15,G=0.15,B=0.15)
		LightShadowMode=LightShadow_ModulateBetter
		ShadowFilterQuality=SFQ_High
		bSynthesizeSHLight=TRUE
	End Object
	Components.Add(MyLightEnvironment)
	LightEnvironment=MyLightEnvironment;

    Begin Object Class=SkeletalMeshComponent Name=InitialSkeletalMesh
		/*AlwaysLoadOnClient=true
		AlwaysLoadOnServer=true
		CastShadow=true
		bCastDynamicShadow=true
		bOwnerNoSee=false
		
		//bUpdateKinematicBonesFromAnimation=true
		//bEnableFullAnimWeightBodies=true
		//TickGroup=TG_PreAsyncWork
		MinDistFactorForKinematicUpdate=0.2
		RBChannel=RBCC_Untitled3
		RBCollideWithChannels=(Untitled3=true)
        BlockRigidBody=true;
        CollideActors=true;
        BlockZeroExtent=true;
		bHasPhysicsAssetInstance = true;*/
		
		bCacheAnimSequenceNodes=FALSE
		AlwaysLoadOnClient=true
		AlwaysLoadOnServer=true
		bOwnerNoSee=false
		CastShadow=true
		BlockRigidBody=TRUE
		bUpdateSkelWhenNotRendered=false
		bIgnoreControllersWhenNotRendered=TRUE
		bUpdateKinematicBonesFromAnimation=true
		bCastDynamicShadow=true
		Translation=(Z=8.0)
		RBChannel=RBCC_Untitled3
		RBCollideWithChannels=(Untitled3=true)
		LightEnvironment=MyLightEnvironment
		bOverrideAttachmentOwnerVisibility=true
		bAcceptsDynamicDecals=FALSE
		bHasPhysicsAssetInstance=true
		bEnableFullAnimWeightBodies=true
		TickGroup=TG_PreAsyncWork
		MinDistFactorForKinematicUpdate=0.2
		bChartDistanceFactor=true
		//bSkipAllUpdateWhenPhysicsAsleep=TRUE
		RBDominanceGroup=20
		Scale=1.075
		MotionBlurScale=0.0
		bAllowAmbientOcclusion=false
		PhysicsAsset=PhysicsAsset'CH_AnimCorrupt.Mesh.SK_CH_Corrupt_Male_Physics'
		AnimSets(0)=AnimSet'CH_AnimHuman.Anims.K_AnimHuman_AimOffset'
		AnimSets(1)=AnimSet'CH_AnimHuman.Anims.K_AnimHuman_BaseMale'
		AnimTreeTemplate=AnimTree'CH_AnimHuman_Tree.AT_CH_Human'
		SkeletalMesh=SkeletalMesh'CH_LIAM_Cathode.Mesh.SK_CH_LIAM_Cathode'
	End Object

	Mesh=InitialSkeletalMesh;
	Components.Add(InitialSkeletalMesh);

	LeftFootControlName=LeftFootControl
	RightFootControlName=RightFootControl;
	bEnableFootPlacement=true;
	MaxFootPlacementDistSquared=56250000.0 // 7500 squared

	CustomGravityScaling=1.0
	SlopeBoostFriction=0.2

	// Default AI values for an EnemyControlled ApocPawn
	Alertness = 1.0f;              //UDK default: 0.0f
	bDontPossess = false;          //UDK default: false
	HearingThreshold = 0.0f;       //UDK default: 2800.0f
	bLOSHearing = false;           //UDK default: true
	bMuffledHearing = false;       //UDK default: false
	PeripheralVision = 0.087f;     //UDK default: 0.0f;
	SightRadius = 5000.0f;         //UDK default: 5000.0f
	
	WeaponTemplate = ApocWeaponScythe'ApocArchetypes.WeaponScythe';
	InventoryManagerClass=class'ApocInventoryManager';
	CustomAnimNodeSlotName=UpperBodyCustomAnim1;

}

event PostBeginPlay()
{
	super.PostBeginPlay();
	// sets up our custom animation cross-blender and two custom animation nodes
	CustomAnimNodeSlot = AnimNodePlayCustomAnim(Mesh.FindAnimNode(CustomAnimNodeSlotName));
	OnPostBeginPlay_AddToTargetableList();

	SetSpeed(MAX_PAWN_SPEED);

	`Log("Custom Pawn up");
}

function SetSpeed(float NewGroundSpeed)
{
	GroundSpeed = NewGroundSpeed;
	AirSpeed = GroundSpeed;
	WaterSpeed = GroundSpeed / 2;
	LadderSpeed = GroundSpeed / 3;
}

function float PlayCustomAnim(name inAnim, optional float playRate = 1.0, optional float blendTimeIn=0.2, optional float blendTimeOut=0.2, optional bool looping=false)
{
	local float PlayTime;
	
	PlayTime = CustomAnimNodeSlot.PlayCustomAnim(inAnim, playRate, blendTimeIn, blendTimeOut, looping, false);
	//SetTimer(PlayTime, false, 'ResetCustomAnim');

	return PlayTime;
}

function bool IsPlayingCustomAnim(name inAnim)
{
	return CustomAnimNodeSlot.bIsPlayingCustomAnim;
}

simulated function name GetDefaultCameraMode( PlayerController RequestedBy )
{
	`Log("Requested Isometric");
    return 'Isometric';
}

simulated event Destroyed()
{
	super.Destroyed();
	OnDestroy_RemoveFromTargetableList();
}

function Ragdollify()
{
	// Ensure we are always updating kinematic
	Mesh.MinDistFactorForKinematicUpdate = 0.0;

	Mesh.ForceSkelUpdate();

	Mesh.SetRBChannel(RBCC_Pawn);
	Mesh.SetRBCollidesWithChannel(RBCC_Default,TRUE);
	Mesh.SetRBCollidesWithChannel(RBCC_Pawn,TRUE);
	Mesh.SetRBCollidesWithChannel(RBCC_Vehicle,TRUE);
	Mesh.SetRBCollidesWithChannel(RBCC_Untitled3,FALSE);
	Mesh.SetRBCollidesWithChannel(RBCC_BlockingVolume,TRUE);


	// Move into post so that we are hitting physics from last frame, rather than animated from this
	Mesh.SetTickGroup(TG_PostAsyncWork);

	PreRagdollCollisionComponent = CollisionComponent;
	CollisionComponent = Mesh;

	// Turn collision on for skelmeshcomp and off for cylinder
	CylinderComponent.SetActorCollision(false, false);
	SetPhysics(PHYS_RigidBody);
	Mesh.PhysicsWeight = 1;
}

function DeRagdollify()
{
	RestorePreRagdollCollisionComponent();	
	CylinderComponent.SetActorCollision(true, true);
	Mesh.PhysicsWeight = 0.f;
	Mesh.PhysicsAssetInstance.SetAllBodiesFixed(TRUE);
	Mesh.bUpdateKinematicBonesFromAnimation=TRUE;
	Mesh.MinDistFactorForKinematicUpdate = default.Mesh.MinDistFactorForKinematicUpdate;
	
	Mesh.SetRBChannel(RBCC_Untitled3);
	Mesh.SetRBCollidesWithChannel(RBCC_Default,FALSE);
	Mesh.SetRBCollidesWithChannel(RBCC_Pawn,FALSE);
	Mesh.SetRBCollidesWithChannel(RBCC_Vehicle,FALSE);
	Mesh.SetRBCollidesWithChannel(RBCC_Untitled3,TRUE);
	Mesh.SetRBCollidesWithChannel(RBCC_BlockingVolume,FALSE);

	if (Physics == PHYS_RigidBody)
	{
		setPhysics(PHYS_Falling);
	}
}

/** Called before the pawn is destroyed */
simulated function PlayDying(class<DamageType> DamageType, vector HitLoc)
{
	Controller.GotoState('Dying');
	Ragdollify();
	super.PlayDying(DamageType, HitLoc);	
	`log(Name @ "Aw hells naw, they killed ma ass!");
	
}

/** Adds a Target to the global targets array in GameInfo, upon creation/PostBeginPlay */
function OnPostBeginPlay_AddToTargetableList()
{
	main(WorldInfo.Game).TargetableActors.AddItem(self);
}

/** Removes a target from the global targets array in GameInfo, upon death/destruction */
function OnDestroy_RemoveFromTargetableList()
{
	main(WorldInfo.Game).TargetableActors.RemoveItem(self);
}

/** Makes it possible to create weapons and such from archetypes */
event final Inventory CreateInventoryFromTemplate( Actor InventoryActorTemplate, optional bool bDoNotActivate )
{
	if ( InvManager != None )
		return ApocInventoryManager(InvManager).CreateInventoryFromTemplate( InventoryActorTemplate, bDoNotActivate );

	return None;
}
