class ApocPeasant extends ApocEnemy
	placeable;
const THIRST_DAMAGE = 5;
const HUNGER_DAMAGE = 1;

var(Apocalypse) ApocPeasantHouse Home;

//Multipliers
var(Apocalypse) float HungerMultiplier;
var(Apocalypse) float ThirstMultiplier;
var(Apocalypse) float CorruptionMultiplier;

//Main stats
//var(Apocalypse) float Health; already defined in PAWN
var(Apocalypse) int Belief;

//Affects the peasants' main stats
var(Apocalypse) float Hunger;
var(Apocalypse) float Thirst;
var(Apocalypse) float Corruption;
var(Apocalypse) int MaxLoad;
var(Apocalypse) int Load;
var(Apocalypse) GoodsType LoadType;

DefaultProperties
{
	HungerMultiplier = 1.0f;
	ThirstMultiplier = 1.0f;
	CorruptionMultiplier = 1.0f;

	Health = 100;
	Belief = 100;

	Hunger = 0.0f;
	Thirst = 0.0f;
	Corruption = 0.0f;

	MaxLoad = 5;
	Load = 0;
}

event bool HealDamage(int Amount, Controller Healer, class<DamageType> DamageType)
{
	UpdateSpeed();
	super.HealDamage(Amount, Healer, DamageType);
}

event TakeDamage(int Damage, Controller InstigatedBy, vector HitLocation, vector Momentum, class<DamageType> DamageType, optional TraceHitInfo HitInfo, optional Actor DamageCauser)
{
	UpdateSpeed();
	super.TakeDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType, HitInfo, DamageCauser);
}

function UpdateSpeed()
{
	SetSpeed(MAX_PAWN_SPEED * float(Health)/float(HealthMax));
}

function AlterThirst(float change)
{
	Thirst += change * ThirstMultiplier;
	Thirst = Max(Thirst, 0.0f);
	if(Thirst > 100.0f)
	{
		Thirst = 100.0f;
		TakeDamage(THIRST_DAMAGE, none, Location, vect(0,0,0), class'DamageType');
	}
}

function AlterHunger(float change)
{
	Hunger += change * HungerMultiplier;
	Hunger = Max(Hunger, 0.0f);
	if(Hunger > 100.0f)
	{
		Hunger = 100.0f;
		TakeDamage(HUNGER_DAMAGE, none, Location,vect(0,0,0), class'DamageType');
	}
}

function AlterCorruption(float change)
{
	Corruption += change * CorruptionMultiplier;
	Corruption = Max(Corruption, 0.0f);
	Corruption = Min(Corruption, 100.0f);
}

function float TimeTilThirsty()
{
	return (100.0f - Thirst) / ThirstMultiplier;
}

function float TimeTilHungry()
{
	return (100.0f - Hunger) / HungerMultiplier;
}

function float EstimatedTimeLeft()
{
	local int EstimatedHealth;
	local float TimeLeft;
	local float TTThirsty;
	local float TTHungry;

	EstimatedHealth = Health;
	TTThirsty = TimeTilThirsty();
	TTHungry = TimeTilHungry();

	if (TTThirsty < TTHungry)
	{
		TimeLeft += TTThirsty;
		TTHungry = TTHungry - TTThirsty;
		if (EstimatedHealth/THIRST_DAMAGE < TTHungry)
		{
			TimeLeft += EstimatedHealth/THIRST_DAMAGE;
			return TimeLeft;
		}
		else
		{
			EstimatedHealth -= TTHungry * THIRST_DAMAGE;
			TimeLeft += TTHungry;
		}
	}
	else
	{
		TimeLeft += TTHungry;
		TTThirsty = TTThirsty - TTHungry;
		if (EstimatedHealth/HUNGER_DAMAGE < TTThirsty)
		{
			TimeLeft += EstimatedHealth/HUNGER_DAMAGE;
			return TimeLeft;
		}
		else
		{
			EstimatedHealth -= TTThirsty * HUNGER_DAMAGE;
			TimeLeft += TTThirsty;
		}
	}
	TimeLeft += EstimatedHealth/(HUNGER_DAMAGE + THIRST_DAMAGE);

	//`log("ApocPeasant:" @ "Estimated time until" @ name @ "dies is:" @ TimeLeft);

	return TimeLeft;
}

function string StatsToString()
{
	return "ApocPeasant:" @ Name @ "Health:" @ Health $ "/" $ HealthMax @ "LoadType:" @ LoadType @ "Load:" @ Load $ "/" $ MaxLoad @ "Thirst:" @ Thirst @ "Water:" @ Home.Water @ "Hunger:" @ Hunger @ "Food:" @ Home.Food @ "Corruption:" @ Corruption;
}
