class ApocPlayerController extends GamePlayerController;

var Vector2D    PlayerMouse;
var Vector      MouseHitWorldLocation;      //Hold where the ray casted from the mouse in 3d coordinate intersect with world geometry. We will
											//use this information for our movement target when not in pathfinding.

var Vector      MouseHitWorldNormal;        //Hold the normalized vector of world location to get direction to MouseHitWorldLocation (calculated in HUD, not used)
var Vector      MousePosWorldLocation;      //Hold deprojected mouse location in 3d world coordinates. (calculated in HUD, not used)
var Vector      MousePosWorldNormal;        //Hold deprojected mouse location normal. (calculated in HUD, used for camera ray from above)

var vector      StartTrace;                 //Hold calculated start of ray from camera
var Vector      EndTrace;                   //Hold calculated end of ray from camera to ground
var vector      RayDir;                     //Hold the direction for the ray query.
var Vector      PawnEyeLocation;            //Hold location of pawn eye for rays that query if an obstacle exist to destination to pathfind.
var Actor       TraceActor;                 //If an actor is found under mouse cursor when mouse moves, its going to end up here.
var vector		AimDirection;				//Holds a vector from the pawn to the cursor.

var(Apocalypse) MeshMouseCursor MouseCursor;
var(Apocalypse) bool FaceCursor;

DefaultProperties
{
	bIsPlayer = true;
}

event Possess(Pawn inPawn, bool bVehicleTransition)
{
	super.Possess(inPawn,bVehicleTransition);
	//Give player weapon
	ApocPlayer(inPawn).CreateInventoryFromTemplate(ApocPlayer(inPawn).WeaponTemplate);
}

simulated event PostBeginPlay()
{
	super.PostBeginPlay();
	MouseCursor = Spawn(class'MeshMouseCursor', self, 'marker');
	`log("ApocPlayerController:" @ "I am alive !");
	
	
	
	//TODO Remy: Legg til i riktig team.
//	ChangeTeam("Horse Men");
//	PlayerReplicationInfo.Team.TeamName = "Horse Men";
//	PlayerReplicationInfo.Team.TeamIndex = 0;
}

event PlayerTick(float DeltaTime)
{
	super.PlayerTick(DeltaTime);
	MouseCursor.SetLocation(MouseHitWorldLocation); //Update mouse cursor

}

state Dead 
{

}

state PlayerWalking{

	/** Primary player movement function */
	function PlayerMove( float DeltaTime )
	{
		local Rotator OldRotation;
		local Rotator CameraRotationYawOnly;
		local Vector PawnRotVector;
		local Vector newAccel;
		local eDoubleClickDir	DoubleClickMove;
		local bool				bSaveJump;

		OldRotation = Rotation;
		DoubleClickMove = DCLICK_None;

		//get camera rotation for camera-relative movement, but only the yaw (we don't care about pitch in this top-down game)
		CameraRotationYawOnly.Yaw = PlayerCamera.ViewTarget.POV.Rotation.Yaw;

		// Update acceleration.
		NewAccel.X = PlayerInput.RawJoyUp;
		NewAccel.Y = PlayerInput.RawJoyRight;
		NewAccel.Z	= 0;

		//transform the acceleration vector by the Camera rotation so that we're doing camera-relative movement
		NewAccel = TransformVectorByRotation(CameraRotationYawOnly,NewAccel);
		//scale it by the pawn's acceleration rate
		NewAccel = Pawn.AccelRate * Normal(NewAccel);

		AimDirection = normal(MouseHitWorldLocation - PawnEyeLocation); //sets the vector to start at the pawns eye and end at the cursor
		PawnRotVector = AimDirection; //Set the local duplicate to Aimdirection.
		PawnRotVector.Z=0; //Remove the Z component.

		if(FaceCursor)
			ApocPlayer(Pawn).UpdateRotation(DeltaTime, PawnRotVector); //Rotates the pawn to face the cursor
		else
			ApocPlayer(Pawn).UpdateRotation(DeltaTime, newAccel); //Rotates the pawn to face acceleration vector

		bDoubleJump = false;

		if( bPressedJump && Pawn.CannotJumpNow() )
		{
			bSaveJump = true;
			bPressedJump = false;
		}
		else
		{
			bSaveJump = false;
		}

		if( Role < ROLE_Authority ) // then save this move and replicate it
		{
			ReplicateMove(DeltaTime, newAccel, DoubleClickMove, OldRotation - Rotation);
		}
		else
		{
			ProcessMove(DeltaTime, newAccel, DoubleClickMove, OldRotation - Rotation);
		}
		bPressedJump = bSaveJump;
	}
}

/*----------------------------------------------------------
 *------------------- CONSOLE FUNCTIONS --------------------
 *----------------------------------------------------------
 */

exec function ToggleFaceCursor()
{
	FaceCursor = !FaceCursor;
}

exec function ZoomOut()
{
	PlayerCamera.FreeCamDistance += 64;	
}

exec function ZoomIn()
{
	PlayerCamera.FreeCamDistance -= 64;
}

exec function Ragdollify()
{
	ApocPawn(Pawn).Ragdollify();
}

exec function DeRagdollify()
{
	ApocPawn(Pawn).DeRagdollify();
}

defaultproperties
{
	CameraClass=class'IsometricCamera'
	FaceCursor = true;
}